/**
 * @author Chenzhyc
 * @description CC视频播放器
 */

import React, { Component } from "react";
import PropTypes from "prop-types";
import appendQuery from "append-query";

const ccPlayerHost = "https://p.bokecc.com/player";
const playerType = "1";

class PBUVideoPlayer extends Component {
	constructor(props) {
		super(props);
		this.state = {
			...props,
		};
		this.playtime = 0; // 播放时间
		this.playingTimer = null; //播放定时器
		this.playRate = 1; // 播放倍速

		this.videoBox = null;
		this.currentTimer = null; //获取实时播放器进度的定时器
	}

	playVideo = () => {
		if (this.getVideoState() == "pause") {
			window.cc_js_Player.play();
		}
	};

	pasueVideo = () => {
		if (this.getVideoState() == "play") {
			window.cc_js_Player.pause();
		}
	};

	jumpToTime = (second) => window.cc_js_Player.jumpToTime(second);
	getPosition = () => window.cc_js_Player.getPosition();
	getCurrRate = () => window.cc_js_Player.getCurrRate();
	getVideoState = () => window.cc_js_Player.getVideoState();

	// 记录视频播放时间
	onListenPlayTime=()=>{
		this.playtime = 0
		clearInterval(this.playingTimer)
		this.playingTimer = setInterval(() => {
			const rate = this.playRate;
			this.playtime = this.playtime + rate
			if(this.playtime > 60){
				this.onCallbackDuration()
				this.onListenPlayTime()
			}
		}, 1000);
	}

	onClosePlayingTimer=()=>{
		clearInterval(this.playingTimer)
	}

	// 每秒回调当前播放进度(关联倍速)
	onListenCurrent = () => {
		clearInterval(this.currentTimer);
		const interval = 1 / this.getCurrRate();
		this.currentTimer = setInterval(() => {
			const time = window.cc_js_Player.getPosition();
			this.props.onRuntimeChange(Number(time.toFixed(1)));
		}, interval * 100);
	};

	// 关闭进度计时器
	onCloseListenCurrent = () => {
		if (this.currentTimer) {
			clearInterval(this.currentTimer);
		}
	};

	// 计算本次播放暂时的时长
	onCallbackDuration() {
		this.props.onRecordPlaytime(Math.ceil(this.playtime));
	}

	loadScript() {
		//先删掉之前的视频播放器节点
		if (this.videoBox.hasChildNodes()) {
			this.videoBox.removeChild(this.videoBox.childNodes[0]);
		}
		window.clearInterval(this.intervalTime);

		const videoSrc = appendQuery(ccPlayerHost, {
			vid: this.state.vid,
			siteid: this.state.siteId,
			autoStart: this.state.autoStart,
			width: this.state.width,
			height: this.state.height,
			playerid: this.state.playerid,
			playertype: playerType,
		});

		const oScript = document.createElement("script");
		oScript.type = "text/javascript";
		oScript.async = true;
		oScript.src = videoSrc;
		this.videoBox.appendChild(oScript);
	}

	componentDidMount() {
		this.loadScript();

		window.on_CCH5player_ready = (obj) => {
			console.log("CCH5player_ready", obj);
			this.playRate = this.getCurrRate();
		};

		window.on_CCH5player_play = () => {
			if(this.props.onRecordPlaytime){
				this.onListenPlayTime()
			}
			if (this.props.onRuntimeChange) {
				this.onListenCurrent();
			}
			if (!this.props.isCanPlayState) {
				this.pasueVideo();
			}
			console.log("CCH5player_play");
		};

		window.on_CCH5player_pause = (obj) => {
			if ("cc_" + this.props.vid != obj.id) {
				return;
			}
			clearInterval(this.playingTimer)
			this.onClosePlayingTimer()
			this.onCloseListenCurrent();
			this.onCallbackDuration();
			console.log("CCH5player_pause");
		};

		window.on_CCH5player_ended = () => {
			this.onClosePlayingTimer()
			this.onCloseListenCurrent();
			this.onCallbackDuration();
			console.log("CCH5player_ended");
		};
		window.get_cc_verification_code = (vid) => {
			return this.props.onGetVerificationCode(vid);
		};

		window.on_switch_rate = (vid, newRate) => {
			this.playRate = newRate;
			if (this.getVideoState() == "play") {
				if (this.props.onRuntimeChange) {
					this.onListenCurrent();
				}
			}
		};
	}

	componentWillReceiveProps(nextProps) {
		this.setState({
			...nextProps,
		});
	}

	shouldComponentUpdate(nextProps) {
		return nextProps.vid !== this.props.vid;
	}

	componentDidUpdate() {
		this.loadScript();
	}

	componentWillUnmount() {
		try {
			if (this.getVideoState() == "play") {
				this.onCallbackDuration();
			}
		} catch (error) {}
		this.onClosePlayingTimer()
	}

	render() {
		return <div ref={(node) => (this.videoBox = node)}></div>;
	}
}

PBUVideoPlayer.propTypes = {
	/**
	 * siteId
	 */
	siteId: PropTypes.string.isRequired,

	/**
	 * 视频唯一标识符
	 */
	vid: PropTypes.string.isRequired,
	/**
	 * 播放器宽度
	 */
	width: PropTypes.string,
	/**
	 * 播放器高度
	 */
	height: PropTypes.string,
	/**
	 * 是否自动播放，默认false
	 */
	autoStart: PropTypes.bool,
	/**
	 * 播放器id，
	 */
	playerid: PropTypes.string,
	/**
	 * @param currentPosition  已播放的秒数
	 */
	onRecordPlaytime: PropTypes.func,
	/**
	 * 获取用于鉴权的验证码
	 * @param {string} vid 视频id
	 * @return {string}
	 */
	onGetVerificationCode: PropTypes.func,
	/**
	 * 定时回调播放时长
	 * @param {number}  间隔时间
	 */
	interval: PropTypes.number,
	// 每秒回调视频播放进度
	onRuntimeChange: PropTypes.func,
	// 拖拽进度条的回调
	onPlayerSeek: PropTypes.func,
	// 当前是否允许播放
	isCanPlayState: PropTypes.bool,
};

PBUVideoPlayer.defaultProps = {
	vid: "",
	siteId: "",
	width: 600,
	height: 490,
	autoStart: false,
	playerid: "",
	interval: 60,
	onGetVerificationCode: (vid) => vid,
	onPlayerSeek: () => {},
	isCanPlayState: true,
};

export default PBUVideoPlayer;
